@@ -1,5 +1,5 @@
 /*
- * Copyright © 2009 Siyan Panayotov <xsisqox@gmail.com>
+ * Copyright © 2009-2010 Siyan Panayotov <xsisqox@gmail.com>
  *
  * Based on code by (see README for details):
  * - Björn Lindqvist <bjourne@gmail.com>
@@ -27,7 +27,6 @@
 
 #include "uni-dragger.h"
 #include "uni-image-view.h"
-#include "mouse_handler.h"
 #include "uni-dragger.h"
 #include "uni-anim-view.h"
 #include "uni-marshal.h"
@@ -461,7 +460,7 @@ uni_image_view_scroll (UniImageView * view,
         ystep = -view->vadj->page_increment;
     else if (yscroll == GTK_SCROLL_PAGE_DOWN)
         ystep = view->vadj->page_increment;
-
+    
     uni_image_view_scroll_to (view,
                               view->offset_x + xstep,
                               view->offset_y + ystep, TRUE, FALSE);
@@ -474,7 +473,7 @@ static void
 uni_image_view_realize (GtkWidget * widget)
 {
     UniImageView *view = UNI_IMAGE_VIEW (widget);
-    GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);
+    gtk_widget_set_realized(widget, TRUE);
 
     GdkWindowAttr attrs;
     attrs.window_type = GDK_WINDOW_CHILD;
@@ -524,7 +523,7 @@ uni_image_view_size_allocate (GtkWidget * widget, GtkAllocation * alloc)
 
     uni_image_view_update_adjustments (view);
 
-    if (GTK_WIDGET_REALIZED (widget))
+    if (gtk_widget_get_realized (widget))
         gdk_window_move_resize (widget->window,
                                 alloc->x, alloc->y,
                                 alloc->width, alloc->height);
@@ -542,7 +541,7 @@ uni_image_view_button_press (GtkWidget * widget, GdkEventButton * ev)
     gtk_widget_grab_focus(widget);
     UniImageView *view = UNI_IMAGE_VIEW (widget);
     VnrWindow *vnr_win = VNR_WINDOW(gtk_widget_get_toplevel(widget));
-    g_assert(GTK_WIDGET_TOPLEVEL(vnr_win));
+    g_assert(gtk_widget_is_toplevel(GTK_WIDGET(vnr_win)));
 
     if(ev->type == GDK_2BUTTON_PRESS && ev->button == 1 && vnr_win->prefs->behavior_click == VNR_PREFS_CLICK_FULLSCREEN)
     {
@@ -564,11 +563,7 @@ uni_image_view_button_press (GtkWidget * widget, GdkEventButton * ev)
     }
     else if (ev->type == GDK_BUTTON_PRESS && ev->button == 1)
     {
-        if(view->vadj->upper > view->vadj->page_size ||
-           view->hadj->upper > view->hadj->page_size)
-            return uni_dragger_button_press (UNI_DRAGGER(view->tool), ev);
-        /* else
-         *     //drag out image */
+        return uni_dragger_button_press (UNI_DRAGGER(view->tool), ev);
     }
     else if (ev->type == GDK_2BUTTON_PRESS && ev->button == 1)
     {
@@ -630,49 +625,38 @@ uni_image_view_scroll_event (GtkWidget * widget, GdkEventScroll * ev)
     gdouble zoom;
     UniImageView *view = UNI_IMAGE_VIEW (widget);
     VnrWindow *vnr_win = VNR_WINDOW(gtk_widget_get_toplevel(widget));
-    g_assert(GTK_WIDGET_TOPLEVEL(vnr_win));
+    g_assert(gtk_widget_is_toplevel(GTK_WIDGET(vnr_win)));
 
     /* Horizontal scroll left is equivalent to scroll up and right is
      * like scroll down. No idea if that is correct -- I have no input
      * device that can do horizontal scrolls. */
-    if (ev->direction == GDK_SCROLL_UP || ev->direction == GDK_SCROLL_LEFT)
-    {
-        if (vnr_win->prefs->behavior_wheel == VNR_PREFS_WHEEL_ZOOM || (ev->state & GDK_CONTROL_MASK) != 0)
-        {
-            zoom = CLAMP (view->zoom * UNI_ZOOM_STEP, UNI_ZOOM_MIN, UNI_ZOOM_MAX);
-            uni_image_view_set_zoom_with_center (view, zoom, ev->x, ev->y, FALSE);
-        }
-        else if(vnr_win->prefs->behavior_wheel == VNR_PREFS_WHEEL_NAVIGATE)
-        {
-            vnr_window_prev(vnr_win);
-        }
-        else
-        {
-            if(ev->direction == GDK_SCROLL_UP)
-                uni_image_view_scroll (view, GTK_SCROLL_NONE, GTK_SCROLL_PAGE_UP);
-            else
-                uni_image_view_scroll (view, GTK_SCROLL_NONE, GTK_SCROLL_PAGE_LEFT);
-        }
-    }
-    else
-    {
-        if (vnr_win->prefs->behavior_wheel == VNR_PREFS_WHEEL_ZOOM || (ev->state & GDK_CONTROL_MASK) != 0)
-        {
-            zoom = CLAMP (view->zoom / UNI_ZOOM_STEP, UNI_ZOOM_MIN, UNI_ZOOM_MAX);
-            uni_image_view_set_zoom_with_center (view, zoom, ev->x, ev->y, FALSE);
-        }
-        else if(vnr_win->prefs->behavior_wheel == VNR_PREFS_WHEEL_NAVIGATE)
-        {
-            vnr_window_next(vnr_win, TRUE);
-        }
-        else
-        {
-            if(ev->direction == GDK_SCROLL_DOWN)
-                uni_image_view_scroll (view,  GTK_SCROLL_NONE, GTK_SCROLL_PAGE_DOWN);
-            else
-                uni_image_view_scroll (view,  GTK_SCROLL_NONE, GTK_SCROLL_PAGE_RIGHT);
-        }
-    }
+    
+	if (vnr_win->prefs->behavior_wheel == VNR_PREFS_WHEEL_ZOOM || (ev->state & GDK_CONTROL_MASK) != 0)
+	{
+		if ( ev->direction == GDK_SCROLL_UP || ev->direction == GDK_SCROLL_LEFT )
+			zoom = CLAMP (view->zoom * UNI_ZOOM_STEP, UNI_ZOOM_MIN, UNI_ZOOM_MAX);
+		else
+			zoom = CLAMP (view->zoom / UNI_ZOOM_STEP, UNI_ZOOM_MIN, UNI_ZOOM_MAX);
+		
+		uni_image_view_set_zoom_with_center (view, zoom, ev->x, ev->y, FALSE);
+	}
+	else if(vnr_win->prefs->behavior_wheel == VNR_PREFS_WHEEL_NAVIGATE)
+	{
+		if (ev->direction == GDK_SCROLL_UP || ev->direction == GDK_SCROLL_LEFT)
+			vnr_window_prev(vnr_win);
+		else
+			vnr_window_next(vnr_win, TRUE);
+	}
+	else
+	{
+		switch (ev->direction) 
+		{
+			case GDK_SCROLL_UP: uni_image_view_scroll (view, GTK_SCROLL_NONE, GTK_SCROLL_PAGE_UP); break;
+			case GDK_SCROLL_LEFT: uni_image_view_scroll (view, GTK_SCROLL_PAGE_LEFT, GTK_SCROLL_NONE); break;
+			case GDK_SCROLL_RIGHT: uni_image_view_scroll (view, GTK_SCROLL_PAGE_RIGHT, GTK_SCROLL_NONE); break;
+			default: uni_image_view_scroll (view, GTK_SCROLL_NONE, GTK_SCROLL_PAGE_DOWN); break;
+		}
+	}
 
     return TRUE;
 }
@@ -711,7 +695,7 @@ uni_image_view_set_scroll_adjustments (UniImageView * view,
 static void
 uni_image_view_init (UniImageView * view)
 {
-    GTK_WIDGET_SET_FLAGS (view, GTK_CAN_FOCUS);
+    gtk_widget_set_can_focus (GTK_WIDGET(view), TRUE);
 
     view->interp = GDK_INTERP_BILINEAR;
     view->fitting = UNI_FITTING_NORMAL;
@@ -1203,6 +1187,7 @@ uni_image_view_set_zoom_mode (UniImageView * view, VnrPrefsZoom mode)
         case VNR_PREFS_ZOOM_SMART:
             uni_image_view_set_fitting(view, UNI_FITTING_NORMAL);
         break;
+        default: break;
     }
 }
 
