@@ -1,5 +1,5 @@
 /*
- * Copyright © 2009 Siyan Panayotov <xsisqox@gmail.com>
+ * Copyright © 2009-2010 Siyan Panayotov <xsisqox@gmail.com>
  *
  * This file is part of Viewnior.
  *
@@ -39,7 +39,7 @@
 #include "vnr-crop.h"
 
 /* Timeout to hide the toolbar in fullscreen mode */
-#define FULLSCREEN_TIMEOUT 3000
+#define FULLSCREEN_TIMEOUT 1000
 
 G_DEFINE_TYPE (VnrWindow, vnr_window, GTK_TYPE_WINDOW);
 
@@ -91,7 +91,7 @@ const gchar *ui_definition = "<ui>"
       "<menuitem name=\"Fullscreen\" action=\"ViewFullscreen\"/>"
       "<menuitem name=\"Slideshow\" action=\"ViewSlideshow\"/>"
       "<separator/>"
-      "<menuitem action=\"ViewResizeWindow\"/>"
+      "<menuitem name=\"ResizeWindow\" action=\"ViewResizeWindow\"/>"
     "</menu>"
     "<menu action=\"Image\">"
       "<menuitem action=\"ImageFlipVertical\"/>"
@@ -130,10 +130,13 @@ const gchar *ui_definition = "<ui>"
     "<menuitem name=\"GoPrevious\" action=\"GoPrevious\"/>"
     "<menuitem name=\"GoNext\" action=\"GoNext\"/>"
     "<separator/>"
+    "<menu action=\"FileOpenWith\">"
+       "<placeholder name=\"AppEntries\"/>"
+    "</menu>"
+    "<separator/>"
     "<menuitem action=\"ViewZoomIn\"/>"
     "<menuitem action=\"ViewZoomOut\"/>"
     "<menuitem action=\"ViewZoomNormal\"/>"
-    "<menuitem action=\"ViewZoomFit\"/>"
     "<placeholder name=\"WallpaperEntry\"/>"
     "<separator/>"
     "<menuitem name=\"Fullscreen\" action=\"ViewFullscreen\"/>"
@@ -261,6 +264,13 @@ vnr_window_update_openwith_menu (VnrWindow *window)
                         name,
                         GTK_UI_MANAGER_MENUITEM,
                         FALSE);
+        gtk_ui_manager_add_ui (window->ui_mngr,
+                        window->open_with_menu_id,
+                        "/PopupMenu/FileOpenWith/AppEntries",
+                        name,
+                        name,
+                        GTK_UI_MANAGER_MENUITEM,
+                        FALSE);
     }
 
     g_list_free (apps);
@@ -289,15 +299,7 @@ vnr_window_load_accel_map()
 static void
 vnr_window_hide_cursor(VnrWindow *window)
 {
-   GdkBitmap *empty;
-   GdkColor black = { 0, 0, 0, 0 };
-   static char bits[] = { 0x00 };
-
-   empty = gdk_bitmap_create_from_data (NULL, bits, 1, 1);
-
-    gdk_window_set_cursor (GTK_WIDGET(window)->window,
-                           gdk_cursor_new_from_pixmap (empty, empty, &black,
-                                                       &black, 0, 0));
+    gdk_window_set_cursor (GTK_WIDGET(window)->window, gdk_cursor_new(GDK_BLANK_CURSOR));
     window->cursor_is_hidden = TRUE;
     gdk_flush();
 }
@@ -315,9 +317,18 @@ update_fs_filename_label(VnrWindow *window)
 {
     if(window->mode == VNR_WINDOW_MODE_NORMAL)
         return;
+        
+    gint position, total;
+    char *buf = NULL;
+    
+    get_position_of_element_in_list(window->file_list, &position, &total);
+    buf = g_strdup_printf ("%s - %i/%i",
+                           VNR_FILE(window->file_list->data)->display_name,
+                           position, total);
 
-    gtk_label_set_text(GTK_LABEL(window->fs_filename_label),
-                       VNR_FILE(window->file_list->data)->display_name);
+    gtk_label_set_text(GTK_LABEL(window->fs_filename_label), buf);
+    
+    g_free(buf);
 }
 
 static gboolean
@@ -422,8 +433,8 @@ get_fs_controls(VnrWindow *window)
 static gboolean
 scrollbars_visible (VnrWindow *window)
 {
-    if (!GTK_WIDGET_VISIBLE (GTK_WIDGET (UNI_SCROLL_WIN(window->scroll_view)->hscroll)) &&
-        !GTK_WIDGET_VISIBLE (GTK_WIDGET (UNI_SCROLL_WIN(window->scroll_view)->vscroll)))
+    if (!gtk_widget_get_visible (GTK_WIDGET (UNI_SCROLL_WIN(window->scroll_view)->hscroll)) &&
+        !gtk_widget_get_visible (GTK_WIDGET (UNI_SCROLL_WIN(window->scroll_view)->vscroll)))
         return FALSE;
 
     return TRUE;
@@ -633,7 +644,10 @@ rotate_pixbuf(VnrWindow *window, GdkPixbufRotation angle)
                               gdk_cursor_new(GDK_WATCH));
     /* This makes the cursor show NOW */
     gdk_flush();
-
+	
+	/* Stop slideshow while editing the image */
+	stop_slideshow(window);
+	
     result = gdk_pixbuf_rotate_simple(UNI_IMAGE_VIEW(window->view)->pixbuf,
                                       angle);
 
@@ -655,7 +669,7 @@ rotate_pixbuf(VnrWindow *window, GdkPixbufRotation angle)
     window->current_image_width = gdk_pixbuf_get_width (result);
     window->current_image_height = gdk_pixbuf_get_height (result);
 
-    if(GTK_WIDGET_VISIBLE(window->props_dlg))
+    if(gtk_widget_get_visible(window->props_dlg))
         vnr_properties_dialog_update_image(VNR_PROPERTIES_DIALOG(window->props_dlg));
 
     /* Extra conditions. Rotating 180 degrees is also flipping horizontal and vertical */
@@ -710,7 +724,7 @@ flip_pixbuf(VnrWindow *window, gboolean horizontal)
 
     uni_anim_view_set_static(UNI_ANIM_VIEW(window->view), result);
 
-    if(GTK_WIDGET_VISIBLE(window->props_dlg))
+    if(gtk_widget_get_visible(window->props_dlg))
         vnr_properties_dialog_update_image(VNR_PROPERTIES_DIALOG(window->props_dlg));
 
     if(!window->cursor_is_hidden)
@@ -782,7 +796,7 @@ fullscreen_motion_cb(GtkWidget * widget, GdkEventMotion * ev, VnrWindow *window)
 
     /* Show the toolbar only when the moves moves to the top
      * of the UniImageView */
-    if (ev->y < 20 && !GTK_WIDGET_VISIBLE (window->toolbar))
+    if (ev->y < 20 && !gtk_widget_get_visible (window->toolbar))
         gtk_widget_show (GTK_WIDGET (window->toolbar));
 
     if(window->cursor_is_hidden)
@@ -894,7 +908,7 @@ save_image_cb (GtkWidget *widget, VnrWindow *window)
     if(window->prefs->behavior_modify != VNR_PREFS_MODIFY_ASK)
         zoom_changed_cb(UNI_IMAGE_VIEW(window->view), window);
 
-    if(GTK_WIDGET_VISIBLE(window->props_dlg))
+    if(gtk_widget_get_visible(window->props_dlg))
         vnr_properties_dialog_update(VNR_PROPERTIES_DIALOG(window->props_dlg));
 }
 
@@ -910,22 +924,49 @@ window_realize_cb(GtkWidget *widget, gpointer user_data)
     g_signal_handlers_disconnect_by_func(widget, window_realize_cb, user_data);
 
     if(!vnr_message_area_is_critical(VNR_MESSAGE_AREA(VNR_WINDOW(widget)->msg_area)))
-    {
-        GdkScreen *screen;
-        GdkRectangle monitor;
-        screen = gtk_window_get_screen (GTK_WINDOW (widget));
-        gdk_screen_get_monitor_geometry (screen,
-                                         gdk_screen_get_monitor_at_window (screen,
-                                            widget->window),
-                                         &monitor);
+    {	
+    	if ( VNR_WINDOW(widget)->prefs->start_maximized ) {
+	        vnr_window_open(VNR_WINDOW(widget), FALSE);
+    	} 
+    	else 
+    	{
+		    GdkScreen *screen;
+		    GdkRectangle monitor;
+		    screen = gtk_window_get_screen (GTK_WINDOW (widget));
+		    gdk_screen_get_monitor_geometry (screen,
+		                                     gdk_screen_get_monitor_at_window (screen,
+		                                        widget->window),
+		                                     &monitor);
 
-        VNR_WINDOW(widget)->max_width = monitor.width * 0.9 - 100;
-        VNR_WINDOW(widget)->max_height = monitor.height * 0.9 - 100;
+		    VNR_WINDOW(widget)->max_width = monitor.width * 0.9 - 100;
+		    VNR_WINDOW(widget)->max_height = monitor.height * 0.9 - 100;
 
-        vnr_window_open(VNR_WINDOW(widget), TRUE);
+		    vnr_window_open(VNR_WINDOW(widget), TRUE);
+		}
+		if ( VNR_WINDOW(widget)->prefs->start_slideshow && VNR_WINDOW(widget)->file_list != NULL ) {
+			vnr_window_fullscreen(VNR_WINDOW(widget));
+			VNR_WINDOW(widget)->mode = VNR_WINDOW_MODE_NORMAL;
+			allow_slideshow(VNR_WINDOW(widget));
+			start_slideshow(VNR_WINDOW(widget));
+		}
     }
 }
 
+static gboolean 
+window_change_state_cb (GtkWidget * widget, GdkEventWindowState * event, gpointer user_data)
+{
+	if ( event->changed_mask & GDK_WINDOW_STATE_MAXIMIZED ) {
+		/* Detect maximized state only */
+		if ( event->new_window_state & GDK_WINDOW_STATE_MAXIMIZED  ) {
+			VNR_WINDOW(widget)->prefs->start_maximized = TRUE;
+		} else {
+			VNR_WINDOW(widget)->prefs->start_maximized = FALSE;
+		}
+		vnr_prefs_save(VNR_WINDOW(widget)->prefs);
+	}
+	return TRUE;
+}
+
 static void
 zoom_changed_cb (UniImageView *view, VnrWindow *window)
 {
@@ -947,6 +988,25 @@ zoom_changed_cb (UniImageView *view, VnrWindow *window)
     }
 }
 
+
+static void
+window_drag_begin_cb (GtkWidget *widget,
+			  GdkDragContext *drag_context,
+			  GtkSelectionData *data,
+			  guint info,
+			  guint time,
+			  gpointer user_data)
+{
+	gchar *uris[2];
+	
+	uris[0] = g_filename_to_uri((gchar*)VNR_FILE(VNR_WINDOW(user_data)->file_list->data)->path, NULL, NULL);
+	uris[1] = NULL;
+
+	gtk_selection_data_set_uris (data, uris);
+	
+	g_free(uris[0]);
+}
+
 static void
 file_open_dialog_response_cb (GtkWidget *dialog,
                               gint response_id,
@@ -970,15 +1030,21 @@ vnr_window_cmd_preferences(GtkAction *action, gpointer user_data)
 }
 
 static void
-vnr_window_cmd_flip_horizontal(GtkAction *action, gpointer user_data)
+vnr_window_cmd_flip_horizontal(GtkAction *action, VnrWindow *window)
 {
-    flip_pixbuf(VNR_WINDOW(user_data), TRUE);
+    if ( !gtk_action_group_get_sensitive(window->actions_static_image) )
+        return;
+	
+    flip_pixbuf(window, TRUE);
 }
 
 static void
-vnr_window_cmd_flip_vertical(GtkAction *action, gpointer user_data)
+vnr_window_cmd_flip_vertical(GtkAction *action, VnrWindow *window)
 {
-    flip_pixbuf(VNR_WINDOW(user_data), FALSE);
+    if ( !gtk_action_group_get_sensitive(window->actions_static_image) )
+        return;
+	
+    flip_pixbuf(window, FALSE);
 }
 
 static void
@@ -1009,6 +1075,7 @@ static void
 vnr_window_cmd_normal_size (GtkAction *action, gpointer user_data)
 {
     uni_image_view_set_zoom(UNI_IMAGE_VIEW(VNR_WINDOW(user_data)->view), 1);
+    uni_image_view_set_fitting(UNI_IMAGE_VIEW(VNR_WINDOW(user_data)->view), UNI_FITTING_NONE);
 }
 
 static void
@@ -1042,8 +1109,11 @@ vnr_window_cmd_prev (GtkAction *action, gpointer user_data)
 }
 
 static void
-vnr_window_cmd_resize (GtkAction *action, VnrWindow *window)
+vnr_window_cmd_resize (GtkToggleAction *action, VnrWindow *window)
 {
+	if ( action != NULL && !gtk_toggle_action_get_active(action) )
+		return;
+	
     gint img_h, img_w;          /* Width and Height of the pixbuf */
 
     img_w = window->current_image_width;
@@ -1149,12 +1219,13 @@ vnr_window_cmd_about (GtkAction *action, VnrWindow *window)
     gtk_show_about_dialog (GTK_WINDOW (window),
                    "program-name", "Viewnior",
                    "version", VERSION,
-                   "copyright", "Copyright \xc2\xa9 2009 Siyan Panayotov <xsisqox@gmail.com>",
+                   "copyright", "Copyright \xc2\xa9 2009-2010 Siyan Panayotov <xsisqox@gmail.com>",
                    "comments",_("Elegant Image Viewer"),
                    "authors", authors,
                    "logo-icon-name", "viewnior",
                    "wrap-license", TRUE,
                    "license", license,
+                   "website", "http://xsisqox.github.com/Viewnior",
                    "translator-credits", _( "translator-credits" ),
                    NULL);
 }
@@ -1328,7 +1399,7 @@ vnr_window_cmd_delete(GtkAction *action, VnrWindow *window)
                 restart_slideshow = FALSE;
 
 
-                if(GTK_WIDGET_VISIBLE(window->props_dlg))
+                if(gtk_widget_get_visible(window->props_dlg))
                     vnr_properties_dialog_clear(VNR_PROPERTIES_DIALOG(window->props_dlg));
             }
             else
@@ -1370,7 +1441,10 @@ static void
 vnr_window_cmd_crop(GtkAction *action, VnrWindow *window)
 {
     VnrCrop *crop;
-
+    
+    if ( !gtk_action_group_get_sensitive(window->actions_static_image) )
+        return;
+		
     crop = (VnrCrop*) vnr_crop_new (window);
 
     if(! vnr_crop_run(crop))
@@ -1401,6 +1475,8 @@ vnr_window_cmd_crop(GtkAction *action, VnrWindow *window)
     window->current_image_width = crop->area.width;
     window->current_image_height = crop->area.height;
 
+    gtk_action_group_set_sensitive(window->action_save, TRUE);
+
     if(window->writable_format_name == NULL)
         vnr_message_area_show(VNR_MESSAGE_AREA(window->msg_area),
                               TRUE,
@@ -1485,9 +1561,6 @@ static const GtkActionEntry action_entries_image[] = {
     { "ViewZoomFit", GTK_STOCK_ZOOM_FIT, N_("Best _Fit"), NULL,
       N_("Fit the image to the window"),
       G_CALLBACK (vnr_window_cmd_fit) },
-    { "ViewResizeWindow", NULL, N_("_Adjust window size"), NULL,
-      N_("Adjust window size to fit the image"),
-      G_CALLBACK (vnr_window_cmd_resize) },
     { "ControlEqual", GTK_STOCK_ZOOM_IN, N_("_Zoom In"), "<control>equal",
       N_("Shrink the image"),
       G_CALLBACK (vnr_window_cmd_zoom_in) },
@@ -1521,6 +1594,9 @@ static const GtkToggleActionEntry toggle_entries_image[] = {
     { "ViewFullscreen", GTK_STOCK_FULLSCREEN, N_("Full _Screen"), "F11",
       N_("Show in fullscreen mode"),
       G_CALLBACK (vnr_window_cmd_fullscreen) },
+    { "ViewResizeWindow", NULL, N_("_Adjust window size"), NULL,
+      N_("Adjust window size to fit the image"),
+      G_CALLBACK (vnr_window_cmd_resize) },
 };
 
 static const GtkToggleActionEntry toggle_entries_window[] = {
@@ -1598,12 +1674,23 @@ vnr_window_key_press (GtkWidget *widget, GdkEventKey *event)
                 GTK_CONTAINER(window->msg_area)->focus_child != NULL)
                 break;
 
+            vnr_window_cmd_next (NULL, window);
+            result = TRUE;
+            break;
+        case GDK_Page_Up:
+            vnr_window_cmd_prev (NULL, window);
+            result = TRUE;
+            break;
+        case GDK_Page_Down:
             vnr_window_cmd_next (NULL, window);
             result = TRUE;
             break;
         case GDK_Escape:
+        case 'q':
             if(window->mode != VNR_WINDOW_MODE_NORMAL)
                 vnr_window_unfullscreen(window);
+            else
+                gtk_main_quit();
             break;
         case GDK_space:
             if (GTK_CONTAINER(window->toolbar)->focus_child != NULL ||
@@ -1616,6 +1703,15 @@ vnr_window_key_press (GtkWidget *widget, GdkEventKey *event)
             vnr_window_prev(window);
             result = TRUE;
             break;
+        case 'h':
+        	vnr_window_cmd_flip_horizontal(NULL, window);
+        	break;
+        case 'v':
+        	vnr_window_cmd_flip_vertical(NULL, window);
+        	break;
+        case 'c':
+        	vnr_window_cmd_crop(NULL, window);
+        	break;
     }
 
     if (result == FALSE && GTK_WIDGET_CLASS (vnr_window_parent_class)->key_press_event)
@@ -1776,7 +1872,7 @@ vnr_window_init (VnrWindow * window)
 
     gtk_action_group_add_toggle_actions (window->action_toolbar,
                                          toggle_entries_window,
-                                         G_N_ELEMENTS (toggle_entries_image),
+                                         G_N_ELEMENTS (toggle_entries_window),
                                          window);
 
     gtk_ui_manager_insert_action_group (window->ui_mngr,
@@ -1885,7 +1981,7 @@ vnr_window_init (VnrWindow * window)
     gtk_widget_show(GTK_WIDGET (window->msg_area));
 
     window->view = uni_anim_view_new ();
-    GTK_WIDGET_SET_FLAGS(window->view, GTK_CAN_FOCUS);
+    gtk_widget_set_can_focus(window->view, TRUE);
     window->scroll_view = uni_scroll_win_new (UNI_IMAGE_VIEW (window->view));
     gtk_box_pack_end (GTK_BOX (window->layout), window->scroll_view, TRUE,TRUE,0);
     gtk_widget_show_all(GTK_WIDGET (window->scroll_view));
@@ -1908,10 +2004,16 @@ vnr_window_init (VnrWindow * window)
 
     g_signal_connect (G_OBJECT (window), "realize",
                       G_CALLBACK (window_realize_cb), NULL);
+                      
+    g_signal_connect (G_OBJECT (window), "window-state-event",
+                      G_CALLBACK (window_change_state_cb), NULL);
 
     g_signal_connect (G_OBJECT (window->view), "zoom_changed",
                       G_CALLBACK (zoom_changed_cb), window);
 
+    g_signal_connect (G_OBJECT (window->view), "drag-data-get",
+                      G_CALLBACK (window_drag_begin_cb), window);
+
     gtk_window_add_accel_group (GTK_WINDOW (window),
                 gtk_ui_manager_get_accel_group (window->ui_mngr));
 
@@ -1927,6 +2029,7 @@ vnr_window_open (VnrWindow * window, gboolean fit_to_screen)
     VnrFile *file;
     GdkPixbufAnimation *pixbuf;
     GdkPixbufFormat *format;
+    UniFittingMode last_fit_mode;
     GError *error = NULL;
 
     if(window->file_list == NULL)
@@ -1943,7 +2046,7 @@ vnr_window_open (VnrWindow * window, gboolean fit_to_screen)
         vnr_message_area_show(VNR_MESSAGE_AREA (window->msg_area),
                               TRUE, error->message, TRUE);
 
-        if(GTK_WIDGET_VISIBLE(window->props_dlg))
+        if(gtk_widget_get_visible(window->props_dlg))
             vnr_properties_dialog_clear(VNR_PROPERTIES_DIALOG(window->props_dlg));
         return FALSE;
     }
@@ -1982,23 +2085,36 @@ vnr_window_open (VnrWindow * window, gboolean fit_to_screen)
 
         gtk_window_resize (GTK_WINDOW (window), img_w, img_h + window->menus->allocation.height);
     }
-
+    
+    last_fit_mode = UNI_IMAGE_VIEW(window->view)->fitting;
+    
     /* Return TRUE if the image is static */
     if ( uni_anim_view_set_anim (UNI_ANIM_VIEW (window->view), pixbuf) )
         gtk_action_group_set_sensitive(window->actions_static_image, TRUE);
     else
         gtk_action_group_set_sensitive(window->actions_static_image, FALSE);
 
-    if(window->mode != VNR_WINDOW_MODE_NORMAL && window->prefs->fit_on_fullscreen)
-        uni_image_view_set_zoom_mode (UNI_IMAGE_VIEW(window->view),
-                                      VNR_PREFS_ZOOM_FIT);
+    if(window->mode != VNR_WINDOW_MODE_NORMAL && window->prefs->fit_on_fullscreen) 
+    {
+		uni_image_view_set_zoom_mode (UNI_IMAGE_VIEW(window->view), VNR_PREFS_ZOOM_FIT);
+    } 
+    else if(window->prefs->zoom == VNR_PREFS_ZOOM_LAST_USED )
+    {
+		uni_image_view_set_fitting (UNI_IMAGE_VIEW(window->view), last_fit_mode);
+		zoom_changed_cb(UNI_IMAGE_VIEW(window->view), window);
+    }
     else
-        uni_image_view_set_zoom_mode (UNI_IMAGE_VIEW(window->view),
-                                      window->prefs->zoom);
-
-    if(GTK_WIDGET_VISIBLE(window->props_dlg))
+    {
+		uni_image_view_set_zoom_mode (UNI_IMAGE_VIEW(window->view), window->prefs->zoom);
+    }
+	
+	if ( gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(gtk_action_group_get_action(window->actions_image, "ViewResizeWindow"))) ) {
+	    vnr_window_cmd_resize(NULL, window);
+	}
+	
+    if(gtk_widget_get_visible(window->props_dlg))
         vnr_properties_dialog_update(VNR_PROPERTIES_DIALOG(window->props_dlg));
-
+    
     vnr_window_update_openwith_menu (window);
 
     g_object_unref(pixbuf);
